@@ -1,6 +1,6 @@
 from __future__ import absolute_import
 
-import os
+import logging
 import select
 import socket
 import ssl
@@ -12,40 +12,47 @@ import vertica_python.vertica.messages as messages
 
 from vertica_python.vertica.messages.message import BackendMessage
 
-from vertica_python.vertica.cursor  import Cursor
+from vertica_python.vertica.cursor import Cursor
+from vertica_python.errors import SSLNotSupported
+
+logger = logging.getLogger('vertica')
+
 
 # To support vertica_python 0.1.9 interface
 class OldResults(object):
     def __init__(self, rows):
         self.rows = rows
 
+
 class Connection(object):
 
-    def __init__(self, options={}):
+    def __init__(self, options=None):
         self.reset_values()
 
-        self.options = {}
+        options = options or {}
+        self.options = {
+            key: value for key, value in options.iteritems() if value is not None
+        }
 
-        for key, value in options.iteritems():
-            if value is not None:
-                self.options[key] = value
-
-        self.options['port'] = 5433 if self.options.get('port') is None else self.options['port']
-        self.options['read_timeout'] = 600 if self.options.get('read_timeout') is None else self.options['read_timeout']
-
-        self.row_style = self.options.get('row_style') if self.options.get('row_style') is not None else 'hash'
+        self.options.setdefault('port', 5433)
+        self.options.setdefault('read_timeout', 600)
         self.boot_connection()
-        #self.debug = True
 
     def __enter__(self):
         return self
 
     def __exit__(self, type, value, traceback):
-        if type:
-            self.rollback()
-        else:
-            self.commit()
+        try:
+            # if there's no outstanding transaction, we can simply close the connection
+            if self.transaction_status in (None, 'in_transaction'):
+                return
 
+            if type is not None:
+                self.rollback()
+            else:
+                self.commit()
+        finally:
+            self.close()
 
     #
     # To support vertica_python 0.1.9 interface
@@ -71,27 +78,25 @@ class Connection(object):
 
     def commit(self):
         if self.closed():
-            raise errors.Error('Connection is closed')
+            raise errors.ConnectionError('Connection is closed')
 
         cur = self.cursor()
         cur.execute('commit')
 
     def rollback(self):
         if self.closed():
-            raise errors.Error('Connection is closed')
+            raise errors.ConnectionError('Connection is closed')
 
         cur = self.cursor()
         cur.execute('rollback')
 
     def cursor(self, cursor_type=None, row_handler=None):
         if self.closed():
-            raise errors.Error('Connection is closed')
+            raise errors.ConnectionError('Connection is closed')
         return Cursor(self, cursor_type=cursor_type, row_handler=row_handler)
 
-
-
     #
-    # Internal 
+    # Internal
     #
 
     def reset_values(self):
@@ -106,7 +111,7 @@ class Connection(object):
         if self.socket is not None:
             return self.socket
 
-        if self.options.get('ssl', False) is True:
+        if self.options.get('ssl'):
             # SSL
             raw_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             raw_socket.connect((self.options['host'], self.options['port']))
@@ -129,7 +134,9 @@ class Connection(object):
         return self.socket is not None and isinstance(ssl.SSLSocket, self.socket)
 
     def opened(self):
-        return self.socket is not None and self.backend_pid is not None and self.transaction_status is not None
+        return (self.socket is not None
+                and self.backend_pid is not None
+                and self.transaction_status is not None)
 
     def closed(self):
         return not self.opened()
@@ -139,8 +146,7 @@ class Connection(object):
         if hasattr(message, 'to_bytes') is False or callable(getattr(message, 'to_bytes')) is False:
             raise TypeError("invalid message: ({0})".format(message))
 
-        if getattr(self, 'debug', False):
-            print "=> {0}".format(message)
+        logger.debug('=> %s', message)
         try:
             self._socket().sendall(message.to_bytes())
         except Exception, e:
@@ -149,8 +155,8 @@ class Connection(object):
 
     def close_socket(self):
         try:
-            self._socket().close()
-            self.socket = None
+            if self.socket is not None:
+                self._socket().close()
         finally:
             self.reset_values()
 
@@ -170,14 +176,17 @@ class Connection(object):
                 size = unpack('!I', self.read_bytes(4))[0]
 
                 if size < 4:
-                    raise errors.MessageError("Bad message size: {0}".format(size))
+                    raise errors.MessageError(
+                        "Bad message size: {0}".format(size)
+                    )
                 message = BackendMessage.factory(type, self.read_bytes(size - 4))
-                if getattr(self, 'debug', False):
-                    print "<= {0}".format(message)
+                logger.debug('<= %s', message)
                 return message
             else:
                 self.close()
                 raise errors.TimedOutError("Connection timed out")
+        except errors.TimedOutError:
+            raise
         except Exception as e:
             self.close_socket()
             raise errors.ConnectionError(e.message)
@@ -199,38 +208,39 @@ class Connection(object):
             raise errors.MessageError("Unhandled message: {0}".format(message))
 
     def __str__(self):
-        safe_options = {}
-        for key, value in self.options.iteritems():
-            if key != 'password':
-                safe_options[key] = value
-        s1 = "<Vertica.Connection:{0} parameters={1} backend_pid={2}, ".format(id(self), self.parameters, self.backend_pid)
-        s2 = "backend_key={0}, transaction_status={1}, socket={2}, options={3}, row_style={4}>".format(self.backend_key, self.transaction_status, self.socket, safe_options, self.row_style)
+        safe_options = {
+            key: value for key, value in self.options.iteritems() if key != 'password'
+        }
+        s1 = "<Vertica.Connection:{0} parameters={1} backend_pid={2}, ".format(
+            id(self), self.parameters, self.backend_pid
+        )
+        s2 = "backend_key={0}, transaction_status={1}, socket={2}, options={3}>".format(
+            self.backend_key, self.transaction_status, self.socket,
+            safe_options,
+        )
         return s1+s2
 
-
-
     def read_bytes(self, n):
         results = ''
         while len(results) < n:
-            bytes = self._socket().recv(n-len(results))
-            if bytes is None or len(bytes) == 0:
+            bytes = self._socket().recv(n - len(results))
+            if not bytes:
                 raise errors.ConnectionError("Connection closed by Vertica")
             results = results + bytes
         return results
 
     def startup_connection(self):
-        if not 'database' in self.options:
-            self.options['database'] = None
-
-        self.write(messages.Startup(self.options['user'], self.options['database']))
-        message = None
+        self.write(messages.Startup(self.options['user'],
+                                    self.options.get('database')))
         while True:
             message = self.read_message()
 
             if isinstance(message, messages.Authentication):
                 # Password message isn't right format ("incomplete message from client")
                 if message.code != messages.Authentication.OK:
-                    self.write(messages.Password(self.options['password'], message.code, dict(user=self.options['user'], salt=getattr(message, 'salt', None))))
+                    self.write(messages.Password(self.options['password'], message.code, {
+                        'user': self.options['user'], 'salt': getattr(message, 'salt', None)
+                    }))
             else:
                 self.process_message(message)
 
@@ -242,7 +252,5 @@ class Connection(object):
             self.query("SET SEARCH_PATH TO {0}".format(self.options['search_path']))
         if self.options.get('role') is not None:
             self.query("SET ROLE {0}".format(self.options['role']))
-        #if self.options.get('interruptable', False) is True:
-        #    self.session_id = self.query("SELECT session_id FROM v_monitor.current_session").the_value()
-
-
+#        if self.options.get('interruptable'):
+#            self.session_id = self.query("SELECT session_id FROM v_monitor.current_session").the_value()
