@@ -99,23 +99,24 @@ class Cursor(object):
             results.append(row)
         return results
 
-
     def nextset(self):
-        raise errors.NotSupportedError('Cursor.nextset() is not implemented')    
+        raise errors.NotSupportedError('Cursor.nextset() is not implemented')
 
     def setinputsizes(self):
         pass
 
-    def setoutputsize(self,size, column=None):
+    def setoutputsize(self, size, column=None):
         pass
 
-
     #
     # Non dbApi methods
     #
     # todo: input stream
     def copy(self, sql, data):
+        # Legacy support
+        self.copy_string(sql, data)
 
+    def _copy_internal(self, sql, datagen):
         if self.closed():
             raise errors.Error('Cursor is closed')
 
@@ -127,18 +128,29 @@ class Cursor(object):
             if isinstance(message, messages.ReadyForQuery):
                 break
             elif isinstance(message, messages.CopyInResponse):
-                #write stuff
-                self.connection.write(messages.CopyData(data))
+                # write stuff
+                for line in datagen:
+                    self.connection.write(messages.CopyData(line))
                 self.connection.write(messages.CopyDone())
 
         if self.error is not None:
             raise self.error
 
+    def copy_string(self, sql, data):
+        self._copy_internal(sql, [data])
+
+
+    def copy_file(self, sql, data, decoder=None):
+        if decoder is None:
+            self._copy_internal(sql, data)
+        else:
+            self._copy_internal(sql, (line.decode(decoder) for line in data))
+
+
     #
     # Internal
     #
 
-
     def closed(self):
         return self._closed or self.connection.closed()
 
@@ -148,7 +160,6 @@ class Cursor(object):
 
         return None
 
-
     def fetch_rows(self):
         while True:
             message = self.connection.read_message()
@@ -156,7 +167,6 @@ class Cursor(object):
             if isinstance(message, messages.ReadyForQuery):
                 break
 
-
     def _process_message(self, message):
         if isinstance(message, messages.ErrorResponse):
             # what am i doing with self.error?
@@ -170,18 +180,16 @@ class Cursor(object):
         elif isinstance(message, messages.DataRow):
             self._handle_datarow(message)
         elif isinstance(message, messages.CommandComplete):
-            #self.result.tag = message.tag
             pass
+#            self.result.tag = message.tag
         else:
             self.connection.process_message(message)
         return None
 
-
     # sets column meta data
     def set_description(self, message):
         self.description = map(lambda fd: Column(fd), message.fields)
 
-
     def _handle_datarow(self, datarow_message):
         row = self.row_formatter(datarow_message)
         if self.row_handler:
@@ -200,37 +208,11 @@ class Cursor(object):
         # throw some error
 
     def format_row_as_dict(self, row_data):
-        row = {}
-        for idx, value in enumerate(row_data.values):
-            col = self.description[idx]
-            row[col.name] = col.convert(value)
-        return row
+        return {
+            self.description[idx].name: self.description[idx].convert(value)
+            for idx, value in enumerate(row_data.values)
+        }
 
     def format_row_as_array(self, row_data):
-        row = []
-        for idx, value in enumerate(row_data.values):
-            col = self.description[idx]
-            row.append(col.convert(value))
-        return row
-
-
-    #COPY_FROM_IO_BLOCK_SIZE = 1024 * 4096
-
-    #def file_copy_handler(self, input_file, output):
-    #    with open(input_file, 'r') as f:
-    #        while True:
-    #            data = f.read(self.COPY_FROM_IO_BLOCK_SIZE)
-    #            if len(data) > 0:
-    #                output.write(data)
-    #            else:
-    #                break
-
-    #def io_copy_handler(self, input, output):
-    #    while True:
-    #        data = input.read(self.COPY_FROM_IO_BLOCK_SIZE)
-    #        if len(data) > 0:
-    #            output.write(data)
-    #        else:
-    #            break
-
-
+        return [self.description[idx].convert(value)
+                for idx, value in enumerate(row_data.values)]
